#include <TFT.h>  
#include <SPI.h>
#include <Wire.h>

#define CS   10
#define DC   9
#define RST  8
#define LEFT 5
#define RIGHT 7
#define OK 6
#define RESET 4 
#define SLAVE_ADDR 4
#define INTERVAL 1000

int leftState, rightState, okState, resetState;
byte colors[7][6];
byte top[7];
byte rows, cols;
int x, y, i, j;
byte currCol, player;
int dirs[8][2];
bool startScreen, gameEnded;
byte choice, timeout;
unsigned long previousMillis;
TFT TFTscreen = TFT(CS, DC, RST);

bool checkWinner() {
  byte inLine, k;
  for (k = 0; k < 8; ++k) {
    i = currCol;
    j = top[currCol];
    inLine = 1;
    while (i + dirs[k][0] >= 0 && i + dirs[k][0] < cols &&
            j + dirs[k][1] >= 0 && j + dirs[k][1] < rows) {
      if (colors[i][j] == colors[i + dirs[k][0]][j + dirs[k][1]]) {
        if (++inLine == 4)
          return true;
      } else
        break;
      i += dirs[k][0];
      j += dirs[k][1];
    }
  }
  return false;
}

void printCol(int col) {
  byte lastCell = 255;
  for (j = 0; j < rows; ++j) {
      if (colors[col][j] != lastCell)
        switch(colors[col][j]) {
          case 1:   TFTscreen.fill(0, 0, 255);
                    lastCell = 1;
                    break;
          case 2:   TFTscreen.fill(0, 255, 255);
                    lastCell = 2;
                    break;
          default:  TFTscreen.fill(255, 255, 255);
                    lastCell = 0;
        }
      TFTscreen.circle(x + 20 * col, y + 20 * j, 8);
  }
}

void setupDirs() {
  dirs[0][0] = 1;
  dirs[0][1] = 1;
  dirs[1][0] = 1; 
  dirs[1][1] = 0;
  dirs[2][0] = 0;
  dirs[2][1] = 1;
  dirs[3][0] = -1;
  dirs[3][1] = -1;
  dirs[4][0] = -1;
  dirs[4][1] = 0;
  dirs[5][0] = 0;
  dirs[5][1] = -1;
  dirs[6][0] = 1;
  dirs[6][1] = -1;
  dirs[7][0] = -1;
  dirs[7][1] = 1;
}

void resetBoard() {
  TFTscreen.background(0, 0, 0);
  TFTscreen.fill(255, 0, 0);
  TFTscreen.rect(8, 4, 142, 120);
  TFTscreen.fill(127, 0, 255);
  TFTscreen.rect(8, 4, 21, 120);
  for (i = 0; i < cols; ++i) {
    for (j = 0; j < rows; ++j)
      colors[i][j] = 0; 
    printCol(i);
    top[i] = rows;
  }
  currCol = 0;
  player = 1;
  Wire.beginTransmission(SLAVE_ADDR);
  Wire.write(4);
  Wire.endTransmission();
}

void resetGame() {
  startScreen = true;
  choice = 1;
  gameEnded = false;
  TFTscreen.background(0, 0, 0);
  TFTscreen.stroke(200, 0, 0);
  TFTscreen.setTextSize(2);
  TFTscreen.text("Choose number", 3, 10);
  TFTscreen.text("of games", 37, 29);
  TFTscreen.stroke(0, 255, 255);
  TFTscreen.text("1/1", 33, 60);
  TFTscreen.stroke(0, 0, 255);
  TFTscreen.text("2/3", 93, 60);
  TFTscreen.text("3/5", 60, 95);
  TFTscreen.stroke(200, 0, 0);
  Wire.beginTransmission(SLAVE_ADDR);
  Wire.write(3);
  Wire.endTransmission();
}

void endScreen() {
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= INTERVAL) {
    previousMillis = currentMillis;
    int redRandom = random(0, 255);
    int greenRandom = random (0, 255);
    int blueRandom = random (0, 255);
    TFTscreen.stroke(redRandom, greenRandom, blueRandom);
    TFTscreen.text("Congrats", 35, 13);
    String txt = "Player ";
    txt.concat(char('0' + player));
    TFTscreen.text(txt.c_str(), 35, 35);
    TFTscreen.text("Press any", 29, 65);
    TFTscreen.text("button", 46, 82);
    TFTscreen.text("to start", 33, 99);
  }
}

void setup() {
  x = 18;
  y = 14;
  rows = 6;
  cols = 7;
  startScreen = true;
  Wire.begin();
  TFTscreen.begin();  
  TFTscreen.background(0, 0, 0);
  resetGame();
  pinMode(LEFT, INPUT);
  digitalWrite(LEFT, HIGH);
  pinMode(RIGHT, INPUT);
  digitalWrite(RIGHT, HIGH);
  pinMode(OK, INPUT);
  digitalWrite(OK, HIGH);
  pinMode(RESET, INPUT);
  digitalWrite(RESET, HIGH);
  setupDirs();
}

void loop() {
  leftState = digitalRead(LEFT);
  rightState = digitalRead(RIGHT);
  okState = digitalRead(OK);
  resetState = digitalRead(RESET);
  if (!gameEnded) {
    if (!startScreen) {
      if (leftState == LOW && currCol > 0) {
        TFTscreen.fill(255, 0, 0);
        TFTscreen.rect(8 + currCol * 20, 4, 21, 120);
        printCol(currCol);
        --currCol;
        if (player == 1)
          TFTscreen.fill(127, 0, 255);
        else
          TFTscreen.fill(0, 204, 102);
        TFTscreen.rect(8 + currCol * 20, 4, 21, 120);
        printCol(currCol);
        delay(350);
      }
    
      if (rightState == LOW && currCol < 6) {
        TFTscreen.fill(255, 0, 0);
        TFTscreen.rect(8 + currCol * 20, 4, 21, 120);
        printCol(currCol);
        ++currCol;
        if (player == 1)
          TFTscreen.fill(127, 0, 255);
        else
          TFTscreen.fill(0, 204, 102);
        TFTscreen.rect(8 + currCol * 20, 4, 21, 120);
        printCol(currCol);
        delay(350);
      }
    
      if ((okState == LOW || timeout) && top[currCol] > 0) {
        --top[currCol];
        for (int j = 0; j < top[currCol]; ++j) {
          if (player == 1)
            TFTscreen.fill(0, 0, 255);
          else
            TFTscreen.fill(0, 255, 255);
          TFTscreen.circle(x + 20 * currCol, y + 20 * j, 8);
          delay(150);
          TFTscreen.fill(255, 255, 255);
          TFTscreen.circle(x + 20 * currCol, y + 20 * j, 8);
        }
        Wire.beginTransmission(SLAVE_ADDR);
        Wire.write(4);
        Wire.endTransmission();
        if (player == 1)
            TFTscreen.fill(0, 0, 255);
          else
            TFTscreen.fill(0, 255, 255);
        colors[currCol][top[currCol]] = player;
        TFTscreen.circle(x + 20 * currCol, y + 20 * top[currCol], 8);
        delay(100);
        if (checkWinner()) {
          Wire.beginTransmission(SLAVE_ADDR);
          Wire.write(player);
          Wire.endTransmission();
          Wire.requestFrom(SLAVE_ADDR, 1);
          byte winner = Wire.read();
          if (!winner) {
            delay(500);
            resetBoard();
          }
          else {
            gameEnded = true;
            TFTscreen.background(0, 0, 0);
          }
        }
        else {
          player = 3 - player;
          delay(70);
          if (player == 1)
            TFTscreen.fill(127, 0, 255);
          else
            TFTscreen.fill(0, 204, 102);
          TFTscreen.rect(8 + currCol * 20, 4, 21, 120);
          printCol(currCol);
        }
        delay(200);
      }
    
      if (resetState == LOW) {
        resetGame();
        delay(200);
      } else {
        Wire.beginTransmission(SLAVE_ADDR);
        Wire.write(5);
        Wire.endTransmission();
        Wire.requestFrom(SLAVE_ADDR, 1);
        timeout = Wire.read();
      }
    } else {
      if (leftState == LOW) {
         switch(choice) {
          case 1:   TFTscreen.stroke(0, 0, 255);
                    TFTscreen.text("1/1", 33, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 2:   TFTscreen.stroke(0, 0, 255);
                    TFTscreen.text("2/3", 93, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 3:   TFTscreen.stroke(0, 0, 255);
                    TFTscreen.text("3/5", 60, 95);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          default:  break;
        }
        --choice;
        if (choice == 0)
          choice = 3;
        switch(choice) {
          case 1:   TFTscreen.stroke(0, 255, 255);
                    TFTscreen.text("1/1", 33, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 2:   TFTscreen.stroke(0, 255, 255);
                    TFTscreen.text("2/3", 93, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 3:   TFTscreen.stroke(0, 255, 255);
                    TFTscreen.text("3/5", 60, 95);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          default:  break;
        }
        delay(350);
      }
      if (rightState == LOW) {
         switch(choice) {
          case 1:   TFTscreen.stroke(0, 0, 255);
                    TFTscreen.text("1/1", 33, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 2:   TFTscreen.stroke(0, 0, 255);
                    TFTscreen.text("2/3", 93, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 3:   TFTscreen.stroke(0, 0, 255);
                    TFTscreen.text("3/5", 60, 95);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          default:  break;
        }
        ++choice;
        if (choice == 4)
          choice = 1;
        switch(choice) {
          case 1:   TFTscreen.stroke(0, 255, 255);
                    TFTscreen.text("1/1", 33, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 2:   TFTscreen.stroke(0, 255, 255);
                    TFTscreen.text("2/3", 93, 60);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          case 3:   TFTscreen.stroke(0, 255, 255);
                    TFTscreen.text("3/5", 60, 95);
                    TFTscreen.stroke(200, 0, 0);
                    break;
          default:  break;
        }
        delay(350);
      }
      if (okState == LOW) {
        startScreen = false;
        resetBoard();
        Wire.beginTransmission(SLAVE_ADDR);
        Wire.write(0);
        Wire.write(choice); 
        Wire.endTransmission();
        delay(300);
      }
    }
  } else {
    endScreen();
    if (okState == LOW || leftState == LOW || rightState == LOW || resetState == LOW) {
      resetGame();
      delay(200);
    }
  }
}
