#include <Wire.h>

#define P2L1 12
#define P2L0 13
#define P1L1 10
#define P1L0 11
#define SLAVE_ADDR 4
#define TIMEOUT_S 10000

byte p1w, p2w, maxw;
unsigned long roundTime;
bool timeoutReq;
byte apas;

void setTimer() {
  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 1999;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS11);
  TIMSK1 |= (1 << OCIE1A);
  sei();
}

void setup() {
  pinMode(P2L0, OUTPUT);
  pinMode(P2L1, OUTPUT);
  pinMode(P1L0, OUTPUT);
  pinMode(P1L1, OUTPUT);
  digitalWrite(P1L0, LOW);
  digitalWrite(P1L1, LOW);
  digitalWrite(P2L0, LOW);
  digitalWrite(P2L1, LOW);
  Wire.begin(SLAVE_ADDR);
  maxw = 10;
  Wire.onRequest(requestEvent);
  Wire.onReceive(receiveEvent);
  setTimer();
}

void loop() {
  if (p1w == maxw) {
      digitalWrite(P1L0, HIGH);
      digitalWrite(P1L1, HIGH);
      delay(400);
      digitalWrite(P1L0, LOW);
      digitalWrite(P1L1, LOW);
      delay(400);
  }
  if (p2w == maxw) {
      digitalWrite(P2L0, HIGH);
      digitalWrite(P2L1, HIGH);
      delay(400);
      digitalWrite(P2L0, LOW);
      digitalWrite(P2L1, LOW);
      delay(400);
  }
}

void requestEvent() {
  if (timeoutReq) {
    if (roundTime >= TIMEOUT_S)
      Wire.write(1);
    else
      Wire.write(0);
    timeoutReq = false;
  } else {
    if (p1w == maxw)
      Wire.write(1);
    else if (p2w == maxw)
      Wire.write(2);
    else
      Wire.write(0);
  }
}

void receiveEvent(int bytes) {
  byte x = Wire.read();
  if (x == 0) {
    maxw = Wire.read();
    roundTime = 0;
  }
  else if (x == 1) {
    ++p1w;
    if (p1w == 1) {
      digitalWrite(P1L0, HIGH);
      digitalWrite(P1L1, LOW);
    } else if (p1w == 2) {
      digitalWrite(P1L0, LOW);
      digitalWrite(P1L1, HIGH);
    } else if (p1w == 3) {
      digitalWrite(P1L0, HIGH);
      digitalWrite(P1L1, HIGH);
    }
  } else if (x == 2) {
    ++p2w;
    if (p2w == 1) {
      digitalWrite(P2L0, HIGH);
      digitalWrite(P2L1, LOW);
    } else if (p2w == 2) {
      digitalWrite(P2L0, LOW);
      digitalWrite(P2L1, HIGH);
    } else if (p2w == 3) {
      digitalWrite(P2L0, HIGH);
      digitalWrite(P2L1, HIGH);
    }
  } else if (x == 3){
    p1w = 0;
    p2w = 0;
    maxw = 10;
    roundTime = 0;
    digitalWrite(P1L0, LOW);
    digitalWrite(P1L1, LOW);
    digitalWrite(P2L0, LOW);
    digitalWrite(P2L1, LOW);
  } else if (x == 4) {
    roundTime = 0;
  } else if (x == 5) {
    timeoutReq = true;
  }
}

ISR(TIMER1_COMPA_vect) {
  ++roundTime;
}
